#define PI 3.141593
#define HIGHLIGHT_TIMEOUT 1000.0 // 1500 ms for highlight effect
precision lowp float;
varying vec2 vTextureCoord;
varying vec2 vTextureCoord2;
uniform vec3 uTileHighlighted;
uniform float uTime;
uniform float uSelected;
uniform float uTileOpacity;
uniform float uTileEdgeGlow;
uniform float uTileExpansionFactor;

varying vec4 vPosition;
varying vec4 vNormal;
varying mat4 vMVPMatrix;

varying vec4 vSpecular;

uniform sampler2D sTexture;

void main() {
    // The frame color/alpha mask
    vec4 glRealColor = texture2D(sTexture, vTextureCoord);

    //
    // Make a version of the pixel with blur. This is a way to add anti aliasing,
    // on the frames that are rotated so that they are not aligned purely
    // vertically or horizontally.
    //

    vec4 glBlurredColor;
    vec4 col1 = glRealColor;

    float dist = 0.005;
    vec2 tex2;
    tex2.x = vTextureCoord.x + dist;
    tex2.y = vTextureCoord.y - dist;
    vec4 col2 = texture2D(sTexture, tex2);

    vec2 tex3;
    tex3.x = vTextureCoord.x - dist;
    tex3.y = vTextureCoord.y - dist;
    vec4 col3 = texture2D(sTexture, tex3);

    vec2 tex4;
    tex4.x = vTextureCoord.x + dist;
    tex4.y = vTextureCoord.y + dist;
    vec4 col4 = texture2D(sTexture, tex4);

    vec2 tex5;
    tex5.x = vTextureCoord.x - dist;
    tex5.y = vTextureCoord.y + dist;
    vec4 col5 = texture2D(sTexture, tex5);

    // Sum the different sampled pixels so make the resulting blur pixel
    glBlurredColor = col1 * 0.5 + col2 * 0.125 + col3 * 0.125 + col4 * 0.125 + col5 * 0.125;

    // Compensate for the fact that the blur makes the pixel a bit darker than the original
    glBlurredColor = glBlurredColor * 1.18;

    //
    // Weigh in the blurred version of the pixel with the non blurred version.
    // uTileExpansionFactor is used here, a tile that is in preview position
    // will not get any blur since it's angled directly towards the camera
    // and therefore doesn't suffer from aliasing effects. It will also look
    // better since the blur that is used for the non preview tiles destroys
    // a bit of the original graphical look.
    //
    vec4 glColor = glBlurredColor * (1.0 - uTileExpansionFactor) + glRealColor * uTileExpansionFactor;

    //
    // Weigh in the tile opacity
    //
    glColor.a = glColor.a * uTileOpacity;

    //
    // Highlight color
    //
    glColor.rgb = glColor.rgb * uTileHighlighted.rgb;

    //
    // Unused variables
    //
    float a = uTime;
    float b = uSelected + uTileOpacity;
    float c = uTileEdgeGlow + uTileExpansionFactor;
    vec2 e = vTextureCoord2;


    //
    // Set the result
    //
    gl_FragColor = glColor;
}
