#define PI 3.141593
#define HIGHLIGHT_TIMEOUT 1000.0 // 1500 ms for highlight effect
precision lowp float;
varying vec2 vTextureCoord;
varying vec2 vTextureCoord2;
uniform float uTime;
uniform sampler2D sTexture;
uniform sampler2D sTexture2;
uniform sampler2D sTexture3;
uniform float uSelected;
uniform float uTileOpacity;
uniform float uTileExpansionFactor;
uniform float uTileReducedExpansionHeightFactor;
uniform float uTileUsingRealTexture;
uniform float uTileInfinityIconPulse;

varying vec4 vSpecular;
varying vec4 vAmbientDiffuseFactor;

const vec4 colorInvisible = vec4(0.0, 0.0, 0.0, 0.0);

void main() {

    //
    // Unused variables
    //
    float a = uTime;
    float d = uSelected;


	//
	// Texture coordinates
	//
	vec2 texChord = vTextureCoord;
    vec2 texChoordOverlay = vTextureCoord2;


    //
    // Calculate coordinates for the bottom dim on the overlay, note this seems to not be part of the current
    // UXC design anymore so the dim is commented out in the tile overlay xml layout file
    //
    float reducedHeight = 1.0 - uTileReducedExpansionHeightFactor;
    texChoordOverlay.y = (texChoordOverlay.y > 0.3) ? (texChoordOverlay.y + reducedHeight) : texChoordOverlay.y;


    //
    // Read from the textures
    //
    vec4 glColor     = texture2D(sTexture,  texChord);
    vec4 glColorMask = texture2D(sTexture2, texChord);
    vec4 glPreviewOverlay = (uTileUsingRealTexture > 0.0) ? texture2D(sTexture3, texChoordOverlay) : colorInvisible;


    //
    // Fade away the edges on the image by setting the alpha for the background image texture to values from the alpha mask texture
    //
    glColor.a = glColorMask.y;


    //
    // Mix in a black overlay layer
    //
    vec4 blackColor = vec4(0.0, 0.0, 0.0, 1.0);
    float amountOfBlack = 0.4;

    glColor.rgb = amountOfBlack * blackColor.a * blackColor.rgb + (1.0 - amountOfBlack) * glColor.a * glColor.rgb;

    float amountOfBlackAlphaToSet = amountOfBlack / 1.5;
    glColor.a = amountOfBlackAlphaToSet * blackColor.a + (1.0 - amountOfBlackAlphaToSet) * glColor.a;


    //
    // Determine whether to show the preview overlay with the infinity icon or not and calculate the infinity icon pulse
    //

    // This is the additional varying glow that will be added to the preview overlay.
    // The parts of the overlay texture that hav an alpha value of less than 95% are
    // the transparent edges of the infinity icon texture. Let the pulse vary only
    // this part. The totally opaque white center of the icon will remain unaffected
    //by the pulse.

    // Define a white color for glow
    vec4 glow = vec4(1.0, 1.0, 1.0, 1.0);

    // Mix in a white glow with the infinity pulse for those pixels that are not totally transparent (those with alpha > 0.2)
    float pulseEffect = ((glPreviewOverlay.a > 0.02) ? uTileInfinityIconPulse : 0.0);
    glPreviewOverlay.rgb = glow.rgb * pulseEffect + glPreviewOverlay.rgb * (1.0 - pulseEffect);

    // If the pulse value is less than -0.9, that means that the pulse has not been correctly determined by the
    // application, in this case we should hide the infinity icon (by multiplying with 0.0)
    glPreviewOverlay = glPreviewOverlay * ((uTileInfinityIconPulse < -0.9) ? 0.0 : 1.0);

    // Use only the upper parts of the preview overlay, those with a texture y-coordinate less than 0.2
    glPreviewOverlay = glPreviewOverlay * ((texChoordOverlay.y < 0.2) ? 1.0 : 0.0);


    //
    // Mix the texture with the preview overlay
    //
    float previewOverlayAlpha = glPreviewOverlay.a * uTileExpansionFactor;
    float previewOverlayMixFactor = previewOverlayAlpha * 0.7;
    glColor.r = previewOverlayMixFactor * glPreviewOverlay.r + (1.0 - previewOverlayMixFactor) * glColor.r;
    glColor.g = previewOverlayMixFactor * glPreviewOverlay.g + (1.0 - previewOverlayMixFactor) * glColor.g;
    glColor.b = previewOverlayMixFactor * glPreviewOverlay.b + (1.0 - previewOverlayMixFactor) * glColor.b;
    glColor.a = max(glColor.a, previewOverlayAlpha) * uTileOpacity;


    //
    // Set the result
    //
    gl_FragColor = glColor;
}
