uniform mat4 uMVPMatrix;
uniform float uSelected;
uniform float uTime;
uniform float uTileOpacity;
uniform float uTileExpansionFactor;
uniform float uTileReducedExpansionHeightFactor;
uniform float uTileUsingRealTexture;
uniform float uTileInfinityIconPulse;
uniform vec3 uLightDirection;
uniform vec4 uLightDiffuse;
uniform vec4 uLightAmbient;
uniform vec4 uLightSpecular;

attribute vec4 aPosition;
attribute vec2 aTextureCoord;
attribute vec2 aTextureCoord2;
attribute vec4 aNormal;

varying vec2 vTextureCoord;
varying vec2 vTextureCoord2;

varying vec4 vPosition;
varying vec4 vNormal;
varying mat4 vMVPMatrix;

varying vec4 vSpecular;
varying vec4 vAmbientDiffuseFactor;

void main() {

	// light ray
    vec3 light = vec3(uLightDirection);
    vec3 mv_light_direction = (uMVPMatrix * vec4(normalize(light), 0.0)).xyz;
    vec3 normal = -normalize(uMVPMatrix *aNormal).xyz;
    vec3 eye = -normalize(uMVPMatrix *aPosition).xyz;
    vec3 reflection = 2.0*reflect(mv_light_direction, normal);

    // diffuse
    vec4 diffuse_factor = max(-dot(normal, mv_light_direction), 0.0) * uLightDiffuse;

    // ambient
    vAmbientDiffuseFactor = diffuse_factor + uLightAmbient;

    // specular
    vec4 specularFact = max(pow(-dot(reflection, eye), 1.0), 0.0) * uLightSpecular;
    vec4 fragSpecular = vec4(0.6,0.6,0.6,0.7);
	vSpecular = specularFact * fragSpecular;

    gl_Position = uMVPMatrix * aPosition;
    vTextureCoord = aTextureCoord;
    vTextureCoord2 = aTextureCoord2;
    vPosition = aPosition;
    vNormal = aNormal;
    vMVPMatrix = uMVPMatrix;
}